@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -23,6 +23,7 @@
 */
 
 #include "pool.h"
+#include "context/pool_session_context.h"
 #include "utils/pool_stream.h"
 #include "pool_config.h"
 #include "auth/pool_passwd.h"
@@ -54,6 +55,8 @@ static int read_password_packet(POOL_CONNECTION *frontend, int protoMajor, 	char
 static int send_password_packet(POOL_CONNECTION *backend, int protoMajor, char *password);
 static int send_auth_ok(POOL_CONNECTION *frontend, int protoMajor);
 
+static long PostmasterRandom(void);
+
 /*
  * After sending the start up packet to the backend, do the
  * authentication against backend. if success return 0 otherwise non
@@ -71,7 +74,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	StartupPacket *sp;
 	
 
-	protoMajor = MAJOR(cp);
+	protoMajor = MASTER_CONNECTION(cp)->sp->major;
 
 	kind = pool_read_kind(cp);
 	if (kind < 0)
@@ -218,13 +221,13 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		if (frontend->auth_method != uaMD5 && !RAW_MODE && NUM_BACKENDS > 1)
 		{
 			pool_send_error_message(frontend, protoMajor, AUTHFAIL_ERRORCODE,
-									"MD5 authentication is unsupported in replication, master-slave and parallel modes.",
+									"MD5 authentication is unsupported in replication and master-slave modes.",
 									"",
 									"check pg_hba.conf",
 									__FILE__, __LINE__);
 			ereport(ERROR,
 				(errmsg("failed to authenticate with backend"),
-					errdetail("MD5 authentication is not supported in replication, master-slave and parallel modes."),
+					errdetail("MD5 authentication is not supported in replication and master-slave modes."),
 						errhint("check pg_hba.conf settings on backend node")));
 		}
 
@@ -405,6 +408,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			strlcpy(cp->info[i].database, sp->database, sizeof(cp->info[i].database));
 			strlcpy(cp->info[i].user, sp->user, sizeof(cp->info[i].user));
 			cp->info[i].counter = 1;
+			CONNECTION(cp, i)->con_info = &cp->info[i];
+			cp->info[i].swallow_termination = 0;
 		}
 	}
 
@@ -1092,6 +1097,10 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 {
 	int length, length0;
 	int i;
+	POOL_SYNC_MAP_STATE use_sync_map = pool_use_sync_map();
+
+	/* Check whether we can use the sync map or not */
+	use_sync_map = pool_use_sync_map();
 
 	/* read message from master node */
 	pool_read(CONNECTION(cp, MASTER_NODE_ID), &length0, sizeof(length0));
@@ -1103,7 +1112,12 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
+		if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i) || use_sync_map == POOL_SYNC_MAP_EMPTY)
+		{
+			continue;
+		}
+
+		if (use_sync_map == POOL_SYNC_MAP_IS_VALID && !pool_is_set_sync_map(i))
 		{
 			continue;
 		}
@@ -1275,13 +1289,51 @@ int pool_read_int(POOL_CONNECTION_POOL *cp)
  */
 void pool_random_salt(char *md5Salt)
 {
-	long rand = random();
+	long rand = PostmasterRandom();
 
 	md5Salt[0] = (rand % 255) + 1;
-	rand = random();
+	rand = PostmasterRandom();
 	md5Salt[1] = (rand % 255) + 1;
-	rand = random();
+	rand = PostmasterRandom();
 	md5Salt[2] = (rand % 255) + 1;
-	rand = random();
+	rand = PostmasterRandom();
 	md5Salt[3] = (rand % 255) + 1;
 }
+
+/*
+ * PostmasterRandom
+ */
+static long
+PostmasterRandom(void)
+{
+	extern struct timeval random_start_time;
+	static unsigned int random_seed = 0;
+
+	/*
+	 * Select a random seed at the time of first receiving a request.
+	 */
+	if (random_seed == 0)
+	{
+		do
+		{
+			struct timeval random_stop_time;
+
+			gettimeofday(&random_stop_time, NULL);
+
+			/*
+			 * We are not sure how much precision is in tv_usec, so we swap
+			 * the high and low 16 bits of 'random_stop_time' and XOR them
+			 * with 'random_start_time'. On the off chance that the result is
+			 * 0, we loop until it isn't.
+			 */
+			random_seed = random_start_time.tv_usec ^
+				((random_stop_time.tv_usec << 16) |
+				 ((random_stop_time.tv_usec >> 16) & 0xffff));
+		}
+		while (random_seed == 0);
+
+		srandom(random_seed);
+	}
+
+	return random();
+}
