@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -27,6 +27,7 @@
 #include "utils/memutils.h"
 #include "utils/elog.h"
 #include "utils/pool_select_walker.h"
+#include "utils/pool_stream.h"
 #include "context/pool_session_context.h"
 #include "context/pool_query_context.h"
 #include "parser/nodes.h"
@@ -87,6 +88,13 @@ void pool_query_context_destroy(POOL_QUERY_CONTEXT *query_context)
 		MemoryContext memory_context = query_context->memory_context;
 		session_context = pool_get_session_context(false);
 		pool_unset_query_in_progress();
+		if (query_context->pg_terminate_backend_conn)
+		{
+			ereport(DEBUG1,
+				 (errmsg("resetting the connection flag for pg_terminate_backend")));
+			pool_unset_connection_will_be_terminated(query_context->pg_terminate_backend_conn);
+		}
+		query_context->pg_terminate_backend_conn = NULL;
 		query_context->original_query = NULL;
 		session_context->query_context = NULL;
 		pfree(query_context);
@@ -190,7 +198,7 @@ void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 			 * send query.
 			 */
 			if (pool_config->master_slave_mode &&
-				!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+				pool_config->master_slave_sub_mode == STREAM_MODE &&
 				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
 			{
 				continue;
@@ -279,7 +287,36 @@ int pool_virtual_master_db_node_id(void)
 
 	if (sc->query_context)
 	{
-		return sc->query_context->virtual_master_node_id;
+		int node_id = sc->query_context->virtual_master_node_id;
+
+		if (STREAM)
+		{
+			 /*
+			  * Make sure that virtual_master_node_id is either primary node
+			  * id or load balance node id.  If not, it is likely that
+			  * virtual_master_node_id is not set up yet. Let's use the
+			  * primary node id. except for the special case where we need
+			  * to send the query to the node which is not primary nor the
+			  * load balance node. Currently there is only one special such
+			  * case that is handling of pg_terminate_backend() function, which
+			  * may refer to the backend connection that is neither hosted by
+			  * the primary or load balance node for current child process, but
+			  * the query must be forwarded to that node. Since only that backend
+			  * node can handle that pg_terminate_backend query
+			  *
+			  */
+			if (node_id != sc->load_balance_node_id && node_id != PRIMARY_NODE_ID)
+			{
+				/*
+				 * Only return the primary node id if we are not processing the
+				 * pg_terminate_backend query
+				 */
+				if (sc->query_context->pg_terminate_backend_conn == NULL)
+					node_id = PRIMARY_NODE_ID;
+			}
+		}
+
+		return node_id;
 	}
 
 	/*
@@ -294,6 +331,28 @@ int pool_virtual_master_db_node_id(void)
 	return my_master_node_id;
 }
 
+/*
+ * The function sets the destination for the current query to the specific backend node
+ */
+void pool_force_query_node_to_backend(POOL_QUERY_CONTEXT *query_context, int backend_id)
+{
+	int i;
+	CHECK_QUERY_CONTEXT_IS_VALID;
+
+	ereport(DEBUG1,
+		(errmsg("forcing query destination node to backend node:%d",backend_id)));
+
+	pool_set_node_to_be_sent(query_context,backend_id);
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		if (query_context->where_to_send[i])
+		{
+			query_context->virtual_master_node_id = i;
+			break;
+		}
+	}
+}
+
 /*
  * Decide where to send queries(thus expecting response)
  */
@@ -409,7 +468,7 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					/*
 					 * If replication delay is too much, we prefer to send to the primary.
 					 */
-					if (!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+					if (pool_config->master_slave_sub_mode == STREAM_MODE &&
 						pool_config->delay_threshold &&
 						bkinfo->standby_delay > pool_config->delay_threshold)
 					{
@@ -479,7 +538,7 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 			}
 		}
 	}
-	else if (REPLICATION || PARALLEL_MODE)
+	else if (REPLICATION)
 	{
 		if (pool_config->load_balance_mode &&
 			is_select_query(node, query) &&
@@ -669,7 +728,7 @@ POOL_STATUS pool_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 		}
 
 		per_node_statement_log(backend, i, string);
-
+		stat_count_up(i, query_context->parse_tree);
 		send_simplequery_message(CONNECTION(backend, i), len, string, MAJOR(backend));
 	}
 
@@ -730,7 +789,7 @@ POOL_STATUS pool_send_and_wait(POOL_QUERY_CONTEXT *query_context,
  */
 POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 										char *kind, int len, char *contents,
-										int send_type, int node_id)
+										int send_type, int node_id, bool nowait)
 {
 	POOL_SESSION_CONTEXT *session_context;
 	POOL_CONNECTION *frontend;
@@ -848,7 +907,36 @@ POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 			per_node_statement_log(backend, i, msgbuf);
 		}
 
+		/* Set sync map so that we could wait for response from appropreate node */
+		pool_set_sync_map(i);
+		ereport(DEBUG1,
+				(errmsg("pool_send_and_wait: pool_set_sync_map: %d", i)));
+
+		/* if Execute message, count up stats count */
+		if (*kind == 'E')
+		{
+			stat_count_up(i, query_context->parse_tree);
+		}
+
 		send_extended_protocol_message(backend, i, kind, str_len, str);
+
+		if ((*kind == 'E' || *kind == 'C') && STREAM)
+		{
+			/*
+			 * Send flush message to backend to make sure that we get any response
+			 * from backend in Sream replication mode.
+			 */
+
+			POOL_CONNECTION *cp = CONNECTION(backend, i);
+			int len;
+
+			pool_write(cp, "H", 1);
+			len = htonl(sizeof(len));
+			pool_write_and_flush(cp, &len, sizeof(len));
+
+			ereport(DEBUG1,
+					(errmsg("pool_send_and_wait: send flush message to %d", i)));
+		}
 	}
 
 	if (!is_begin_read_write)
@@ -859,47 +947,50 @@ POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 			str = query_context->original_query;
 	}
 
-	/* Wait for response */
-	for (i=0;i<NUM_BACKENDS;i++)
+	if (!nowait)
 	{
-		if (!VALID_BACKEND(i))
-			continue;
-		else if (send_type < 0 && i == node_id)
-			continue;
-		else if (send_type > 0 && i != node_id)
-			continue;
-
-		/*
-		 * If in master/slave mode, we do not send COMMIT/ABORT to
-		 * slaves/standbys if it's in I(idle) state.
-		 */
-		if (is_commit && MASTER_SLAVE && !IS_MASTER_NODE_ID(i) && TSTATE(backend, i) == 'I')
+		/* Wait for response */
+		for (i=0;i<NUM_BACKENDS;i++)
 		{
-			continue;
+			if (!VALID_BACKEND(i))
+				continue;
+			else if (send_type < 0 && i == node_id)
+				continue;
+			else if (send_type > 0 && i != node_id)
+				continue;
+
+			/*
+			 * If in master/slave mode, we do not send COMMIT/ABORT to
+			 * slaves/standbys if it's in I(idle) state.
+			 */
+			if (is_commit && MASTER_SLAVE && !IS_MASTER_NODE_ID(i) && TSTATE(backend, i) == 'I')
+			{
+				continue;
+			}
+
+			if (is_begin_read_write)
+			{
+				if (REAL_PRIMARY_NODE_ID == i)
+					str = query_context->original_query;
+				else
+					str = query_context->rewritten_query;
+			}
+
+			wait_for_query_response_with_trans_cleanup(frontend,
+													   CONNECTION(backend, i),
+													   MAJOR(backend),
+													   MASTER_CONNECTION(backend)->pid,
+													   MASTER_CONNECTION(backend)->key);
+
+			/*
+			 * Check if some error detected.  If so, emit
+			 * log. This is useful when invalid encoding error
+			 * occurs. In this case, PostgreSQL does not report
+			 * what statement caused that error and make users
+			 * confused.
+			 */		
+			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend: ", true);
 		}
-
-		if (is_begin_read_write)
-		{
-			if (REAL_PRIMARY_NODE_ID == i)
-				str = query_context->original_query;
-			else
-				str = query_context->rewritten_query;
-		}
-
-        wait_for_query_response_with_trans_cleanup(frontend,
-                                                   CONNECTION(backend, i),
-                                                   MAJOR(backend),
-                                                   MASTER_CONNECTION(backend)->pid,
-                                                   MASTER_CONNECTION(backend)->key);
-
-		/*
-		 * Check if some error detected.  If so, emit
-		 * log. This is useful when invalid encoding error
-		 * occurs. In this case, PostgreSQL does not report
-		 * what statement caused that error and make users
-		 * confused.
-		 */		
-		per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend: ", true);
 	}
 
 	if(rewritten_begin)
@@ -929,7 +1020,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 #define AccessExclusiveLock		8		/* ALTER TABLE, DROP TABLE, VACUUM
 										 * FULL, and unqualified LOCK TABLE */
 
-/* From 9.0 include/nodes/node.h */
+/* From 9.5 include/nodes/node.h ("TAGS FOR STATEMENT NODES" part) */
 	static NodeTag nodemap[] = {
 		T_PlannedStmt,
 		T_InsertStmt,
@@ -942,9 +1033,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		T_SetOperationStmt,
 		T_GrantStmt,
 		T_GrantRoleStmt,
-		/*
-		T_AlterDefaultPrivilegesStmt,	Our parser does not support yet
-		*/
+		T_AlterDefaultPrivilegesStmt,
 		T_ClosePortalStmt,
 		T_ClusterStmt,
 		T_CopyStmt,
@@ -957,9 +1046,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		T_IndexStmt,	/* CREATE INDEX */
 		T_CreateFunctionStmt,
 		T_AlterFunctionStmt,
-		/*
-		T_DoStmt,		Our parser does not support yet
-		*/
+		T_DoStmt,
 		T_RenameStmt,	/* ALTER AGGREGATE etc. */
 		T_RuleStmt,		/* CREATE RULE */
 		T_NotifyStmt,
@@ -973,6 +1060,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		T_DropdbStmt,
 		T_VacuumStmt,
 		T_ExplainStmt,
+		T_CreateTableAsStmt,
 		T_CreateSeqStmt,
 		T_AlterSeqStmt,
 		T_VariableSetStmt,		/* SET */
@@ -1008,6 +1096,8 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		T_ReassignOwnedStmt,
 		T_CompositeTypeStmt,	/* CREATE TYPE */
 		T_CreateEnumStmt,
+		T_CreateRangeStmt,
+		T_AlterEnumStmt,
 		T_AlterTSDictionaryStmt,
 		T_AlterTSConfigurationStmt,
 		T_CreateFdwStmt,
@@ -1017,9 +1107,22 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		T_CreateUserMappingStmt,
 		T_AlterUserMappingStmt,
 		T_DropUserMappingStmt,
-		/*
-		T_AlterTableSpaceOptionsStmt,	Our parser does not support yet
-		*/
+		T_AlterTableSpaceOptionsStmt,
+		T_AlterTableMoveAllStmt,
+		T_SecLabelStmt,
+		T_CreateForeignTableStmt,
+		T_ImportForeignSchemaStmt,
+		T_CreateExtensionStmt,
+		T_AlterExtensionStmt,
+		T_AlterExtensionContentsStmt,
+		T_CreateEventTrigStmt,
+		T_AlterEventTrigStmt,
+		T_RefreshMatViewStmt,
+		T_ReplicaIdentityStmt,
+		T_AlterSystemStmt,
+		T_CreatePolicyStmt,
+		T_AlterPolicyStmt,
+		T_CreateTransformStmt,
 	};
 
 	if (bsearch(&nodeTag(node), nodemap, sizeof(nodemap)/sizeof(nodemap[0]),
@@ -1035,15 +1138,36 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			if (pool_has_insertinto_or_locking_clause(node))
 				return POOL_PRIMARY;
 
+			/* non-SELECT query in WITH clause ? */
+			if (((SelectStmt *)node)->withClause)
+			{
+				List *ctes = ((SelectStmt *)node)->withClause->ctes;
+				ListCell   *cte_item;
+				foreach(cte_item, ctes)
+				{
+					CommonTableExpr *cte = (CommonTableExpr *)lfirst(cte_item);
+					if (!IsA(cte->ctequery, SelectStmt))
+						return POOL_PRIMARY;
+				}
+			}
+
 			return POOL_EITHER;
 		}
 
 		/*
-		 * COPY FROM
+		 * COPY
 		 */
 		else if (IsA(node, CopyStmt))
 		{
-			return (((CopyStmt *)node)->is_from)?POOL_PRIMARY:POOL_EITHER;
+			if (((CopyStmt *)node)->is_from)
+				return POOL_PRIMARY;
+			else
+			{
+				if (((CopyStmt *)node)->query == NULL)
+					return POOL_EITHER;
+				else
+					return (IsA(((CopyStmt *)node)->query, SelectStmt))?POOL_EITHER:POOL_PRIMARY;
+			}
 		}
 
 		/*
@@ -1289,7 +1413,8 @@ char *pool_get_query_string(void)
 }
 
 /*
- * Return true if the query is:
+ * Returns true if the query is one of:
+ *
  * SET TRANSACTION ISOLATION LEVEL SERIALIZABLE or
  * SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL SERIALIZABLE or
  * SET transaction_isolation TO 'serializable'
@@ -1303,7 +1428,8 @@ bool is_set_transaction_serializable(Node *node)
 		return false;
 
 	if (((VariableSetStmt *)node)->kind == VAR_SET_VALUE &&
-		!strcmp(((VariableSetStmt *)node)->name, "transaction_isolation"))
+		(!strcmp(((VariableSetStmt *)node)->name, "transaction_isolation") ||
+                 !strcmp(((VariableSetStmt *)node)->name, "default_transaction_isolation")))
 	{
 		List *options = ((VariableSetStmt *)node)->args;
 		foreach(list_item, options)
@@ -1647,3 +1773,77 @@ void pool_unset_cache_exceeded(void)
 		sc->query_context->temp_cache->is_exceeded = false;
 	}
 }
+
+/*
+ * Return true if one of followings is true
+ *
+ * SET transaction_read_only TO on
+ * SET TRANSACTION READ ONLY
+ * SET TRANSACTION CHARACTERISTICS AS TRANSACTION READ ONLY
+ *
+ * Note that if the node is not a variable statement, returns false.
+ */
+bool pool_is_transaction_read_only(Node *node)
+{
+	ListCell   *list_item;
+	bool ret = false;
+
+	if (!IsA(node, VariableSetStmt))
+		return ret;
+
+	/*
+	 * SET transaction_read_only TO on
+	 */
+	if (((VariableSetStmt *)node)->kind == VAR_SET_VALUE &&
+		!strcmp(((VariableSetStmt *)node)->name, "transaction_read_only"))
+	{
+		List *options = ((VariableSetStmt *)node)->args;
+		foreach(list_item, options)
+		{
+			A_Const *v = (A_Const *)lfirst(list_item);
+
+			switch (v->val.type)
+			{
+				case T_String:
+					if (!strcasecmp(v->val.val.str, "on") ||
+						!strcasecmp(v->val.val.str, "t") ||
+						!strcasecmp(v->val.val.str, "true"))
+						ret = true;
+					break;
+				case T_Integer:
+					if (v->val.val.ival)
+						ret = true;
+				default:
+					break;
+			}
+		}
+	}
+
+	/*
+	 * SET SESSION CHARACTERISTICS AS TRANSACTION READ ONLY
+	 * SET TRANSACTION READ ONLY
+	 */
+	else if (((VariableSetStmt *)node)->kind == VAR_SET_MULTI &&
+			 (!strcmp(((VariableSetStmt *)node)->name, "TRANSACTION") ||
+			  !strcmp(((VariableSetStmt *)node)->name, "SESSION CHARACTERISTICS")))
+	{
+		List *options = ((VariableSetStmt *)node)->args;
+		foreach(list_item, options)
+		{
+			DefElem *opt = (DefElem *) lfirst(list_item);
+
+			if (!strcmp("transaction_read_only", opt->defname))
+			{
+				bool read_only;
+
+				read_only = ((A_Const *)opt->arg)->val.val.ival;
+				if (read_only)
+				{
+					ret = true;
+					break;
+				}
+			}
+		}
+	}
+	return ret;
+}
