@@ -77,6 +77,9 @@
 /* Unix domain socket directory */
 #define DEFAULT_SOCKET_DIR "/tmp"
 
+/* Unix domain socket directory for watchdog IPC */
+#define DEFAULT_WD_IPC_SOCKET_DIR "/tmp"
+
 /* pid file name */
 #define DEFAULT_PID_FILE_NAME "/var/run/pgpool/pgpool.pid"
 
@@ -101,6 +104,13 @@ typedef enum {
 	POOL_DEADLOCK
 } POOL_STATUS;
 
+typedef enum {
+	POOL_SOCKET_CLOSED = 0,
+	POOL_SOCKET_VALID,
+	POOL_SOCKET_ERROR,
+	POOL_SOCKET_EOF
+} POOL_SOCKET_STATE;
+
 /* protocol major version numbers */
 #define PROTO_MAJOR_V2	2
 #define PROTO_MAJOR_V3	3
@@ -137,7 +147,6 @@ typedef struct
 	char *database;	/* database name in startup_packet (malloced area) */
 	char *user;	/* user name in startup_packet (malloced area) */
 	char *application_name;		/* not malloced. pointing to in startup_packet */
-    bool system_db;     /* true if database name is one of default that comes with server e.g postgres or template.. */
 } StartupPacket;
 
 typedef struct CancelPacket
@@ -216,7 +225,7 @@ typedef struct {
 	char kind;	/* kind cache */
 
 	/* true if remote end closed the connection */
-	bool EOF_on_socket;
+	POOL_SOCKET_STATE socket_state;
 	/*
 	 * frontend info needed for hba
 	 */
@@ -226,6 +235,9 @@ typedef struct {
 	char *auth_arg;
 	char *database;
 	char *username;
+	ConnectionInfo *con_info; /* shared memory coninfo used
+						   * for handling the query containing
+						   * pg_terminate_backend*/
 } POOL_CONNECTION;
 
 /*
@@ -246,14 +258,6 @@ typedef struct {
     POOL_CONNECTION_POOL_SLOT	*slots[MAX_NUM_BACKENDS];
 } POOL_CONNECTION_POOL;
 
-typedef struct {
-	SystemDBInfo *info;
-	PGconn *pgconn;
-	/* persistent connection to the system DB */
-	POOL_CONNECTION_POOL_SLOT *connection;
-	BACKEND_STATUS *system_db_status;
-} POOL_SYSTEMDB_CONNECTION_POOL;
-
 /* 
  * for pool_clear_cache() in pool_query_cache.c 
  *
@@ -322,6 +326,7 @@ extern int my_master_node_id;
  */
 #define PRIMARY_NODE_ID (Req_info->primary_node_id >=0?\
 						 Req_info->primary_node_id:REAL_MASTER_NODE_ID)
+#define IS_PRIMARY_NODE_ID(node_id)	(node_id == PRIMARY_NODE_ID)
 
 /*
  * Real primary node id. If not in the mode or there's no primary
@@ -341,16 +346,13 @@ extern int my_master_node_id;
 
 #define REPLICATION (pool_config->replication_mode)
 #define MASTER_SLAVE (pool_config->master_slave_mode)
-#define STREAM (MASTER_SLAVE && !strcmp("stream", pool_config->master_slave_sub_mode))
+#define STREAM (MASTER_SLAVE && pool_config->master_slave_sub_mode == STREAM_MODE)
+#define SLONY (MASTER_SLAVE && pool_config->master_slave_sub_mode == SLONY_MODE)
 #define DUAL_MODE (REPLICATION || MASTER_SLAVE)
-#define PARALLEL_MODE (pool_config->parallel_mode)
-#define RAW_MODE (!REPLICATION && !PARALLEL_MODE && !MASTER_SLAVE)
-#define MAJOR(p) MASTER_CONNECTION(p)->sp->major
+#define RAW_MODE (!REPLICATION && !MASTER_SLAVE)
+#define MAJOR(p) (pool_get_major_version())
 #define TSTATE(p, i) (CONNECTION(p, i)->tstate)
 #define INTERNAL_TRANSACTION_STARTED(p, i) (CONNECTION(p, i)->is_internal_transaction_started)
-#define SYSDB_INFO (system_db_info->info)
-#define SYSDB_CONNECTION (system_db_info->connection)
-#define SYSDB_STATUS (*system_db_info->system_db_status)
 
 #define Max(x, y)		((x) > (y) ? (x) : (y))
 #define Min(x, y)		((x) < (y) ? (x) : (y))
@@ -362,13 +364,22 @@ extern int my_master_node_id;
 #define NO_LOAD_BALANCE "/*NO LOAD BALANCE*/"
 #define NO_LOAD_BALANCE_COMMENT_SZ (sizeof(NO_LOAD_BALANCE)-1)
 
-#define MAX_NUM_SEMAPHORES		4
-#define CONN_COUNTER_SEM 0
-#define REQUEST_INFO_SEM 1
-#define SHM_CACHE_SEM	2
+#define MAX_NUM_SEMAPHORES		6
+#define CONN_COUNTER_SEM		0
+#define REQUEST_INFO_SEM		1
+#define SHM_CACHE_SEM			2
 #define QUERY_CACHE_STATS_SEM	3
+#define PCP_REQUEST_SEM			4
+#define ACCEPT_FD_SEM			5
 #define MAX_REQUEST_QUEUE_SIZE	10
 
+#define MAX_SEC_WAIT_FOR_CLUSTER_TRANSATION 10 /* time in seconds to keep retrying for a
+											   * watchdog command if the cluster is not
+											   * in stable state */
+
+#define SERIALIZE_ACCEPT (pool_config->serialize_accept == true && \
+						  pool_config->child_life_time == 0)
+
 /*
  * number specified when semaphore is locked/unlocked
  */
@@ -390,8 +401,12 @@ typedef enum {
 	PROMOTE_NODE_REQUEST
 } POOL_REQUEST_KIND;
 
+#define REQ_DETAIL_SWITCHOVER	0x00000001		/* failover due to switch over */
+
 typedef struct {
 	POOL_REQUEST_KIND	kind;		/* request kind */
+	unsigned char request_details;	/* option flags kind */
+	unsigned int wd_failover_id;	/* watchdog ID for this failover operation */
 	int node_id[MAX_NUM_BACKENDS];	/* request node id */
 	int count;						/* request node ids count */
 }POOL_REQUEST_NODE;
@@ -442,7 +457,6 @@ typedef enum {
  * global variables
  */
 extern pid_t mypid; /* parent pid */
-extern bool run_as_pcp_child;
 
 typedef enum
 {
@@ -455,7 +469,8 @@ typedef enum
 	PT_LIFECHECK,
 	PT_FOLLOWCHILD,
 	PT_WATCHDOG_UTILITY,
-	PT_PCP
+	PT_PCP,
+	PT_PCP_WORKER
 } ProcessType;
 
 extern ProcessType processType;
@@ -464,7 +479,6 @@ typedef enum
 {
 	INITIALIZING,
 	PERFORMING_HEALTH_CHECK,
-	PERFORMING_SYSDB_CHECK,
 	SLEEPING,
 	WAITIG_FOR_CONNECTION,
 	BACKEND_CONNECTING,
@@ -479,7 +493,6 @@ extern volatile sig_atomic_t backend_timer_expired; /* flag for connection close
 extern volatile sig_atomic_t health_check_timer_expired;		/* non 0 if health check timer expired */
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 extern int my_proc_id;  /* process table id (!= UNIX's PID) */
-extern POOL_SYSTEMDB_CONNECTION_POOL *system_db_info; /* systemdb */
 extern ProcessInfo *process_info; /* shmem process information table */
 extern ConnectionInfo *con_info; /* shmem connection info table */
 extern POOL_REQUEST_INFO *Req_info;
@@ -488,6 +501,7 @@ extern char remote_ps_data[];		/* used for set_ps_display */
 extern volatile sig_atomic_t got_sighup;
 extern volatile sig_atomic_t exit_request;
 
+
 #define QUERY_STRING_BUFFER_LEN 1024
 extern char query_string_buffer[];		/* last query string sent to simpleQuery() */
 
@@ -499,11 +513,11 @@ extern char remote_port[];	/* client port */
 /*
  * public functions
  */
-extern bool register_node_operation_request(POOL_REQUEST_KIND kind, int* node_id_set, int count);
+extern bool register_node_operation_request(POOL_REQUEST_KIND kind, int* node_id_set, int count, bool switch_over, unsigned int wd_failover_id);
 extern char *get_config_file_name(void);
 extern char *get_hba_file_name(void);
 extern void do_child(int *fds);
-extern void pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file);
+extern void pcp_main(int unix_fd, int inet_fd);
 extern int select_load_balancing_node(void);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
@@ -513,6 +527,8 @@ extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 extern int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 extern int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp);
 
+extern bool is_backend_cache_empty(POOL_CONNECTION_POOL *backend);
+
 /* SSL functionality */
 extern void pool_ssl_negotiate_serverclient(POOL_CONNECTION *cp);
 extern void pool_ssl_negotiate_clientserver(POOL_CONNECTION *cp);
@@ -527,11 +543,11 @@ extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend,
 extern void NoticeResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend);
 
-extern void notice_backend_error(int node_id);
-extern void degenerate_backend_set(int *node_id_set, int count);
-extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool error, bool test_only);
-extern void promote_backend(int node_id);
-extern void send_failback_request(int node_id);
+extern void notice_backend_error(int node_id, bool switch_over);
+extern void degenerate_backend_set(int *node_id_set, int count, bool switch_over, unsigned int wd_failover_id);
+extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool error, bool test_only, bool switch_over, unsigned int wd_failover_id);
+extern void promote_backend(int node_id, unsigned int wd_failover_id);
+extern void send_failback_request(int node_id, bool throw_error, unsigned int wd_failover_id);
 
 
 extern void pool_set_timeout(int timeoutval);
@@ -606,16 +622,7 @@ extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection_noerror(
 extern void discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT *cp);
 
 /* define pool_system.c */
-extern POOL_CONNECTION_POOL_SLOT *pool_system_db_connection(void);
-extern DistDefInfo *pool_get_dist_def_info (char * dbname, char * schema_name, char * table_name);
-extern RepliDefInfo *pool_get_repli_def_info (char * dbname, char * schema_name, char * table_name);
-extern int pool_get_id (DistDefInfo *info, const char * value);
-extern int system_db_connect (void);
-extern int pool_memset_system_db_info (SystemDBInfo *info);
 extern void pool_close_libpq_connection(void);
-extern int system_db_health_check(void);
-extern SystemDBInfo *pool_get_system_db_info(void);
-
 
 /* pool_hba.c */
 extern int load_hba(char *hbapath);
@@ -694,7 +701,27 @@ extern int connect_inet_domain_socket_by_port(char *host, int port, bool retry);
 extern int connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry);
 extern int pool_pool_index(void);
 
+/* utils/statistics.c */
+size_t stat_shared_memory_size(void);
+void stat_set_stat_area(void *address);
+void stat_init_stat_area(void);
+void stat_count_up(int backend_node_id, Node *parsetree);
+uint64 stat_get_select_count(int backend_node_id);
+
 extern int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps);
+
+/* pcp_child.c */
+extern int send_to_pcp_frontend(char* data, int len, bool flush);
+extern int pcp_frontend_exists(void);
+extern void pcp_worker_main(int port);
+extern void pcp_mark_recovery_finished(void);
+extern bool pcp_mark_recovery_in_progress(void);
+
+
+/* pgpool_main.c */
+extern int pool_send_to_frontend(char* data, int len, bool flush);
+extern int pool_frontend_exists(void);
+extern pid_t pool_waitpid(int *status);
 extern int write_status_file(void);
 
 #endif /* POOL_H */
