@@ -95,7 +95,7 @@ pcp_worker_main(int port)
 {
 	sigjmp_buf	local_sigjmp_buf;
 	MemoryContext PCPMemoryContext;
-	int authenticated = 0;
+	volatile int authenticated = 0;
 
 	char salt[4];
 	int random_salt = 0;
@@ -502,14 +502,14 @@ static int pool_detach_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		degenerate_backend_set_ex(&node_id, 1, true, false);
+		degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
 		return 0;
 	}
 
 	/* Check if the NODE DOWN can be executed on
 	 * the given node id.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, true);
+	degenerate_backend_set_ex(&node_id, 1, true, true, true, 0);
 
 	/*
 	 * Wait until all frontends exit
@@ -529,7 +529,7 @@ static int pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, false);
+	degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
 
 	/*
 	 * Wait for failover completed.
@@ -556,7 +556,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		promote_backend(node_id);	/* send promote request */
+		promote_backend(node_id, false);	/* send promote request */
 		return 0;
 	}
 
@@ -576,7 +576,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	promote_backend(node_id);		/* send promote request */
+	promote_backend(node_id, false);		/* send promote request */
 
 	/*
 	 * Wait for failover completed.
@@ -910,7 +910,7 @@ process_attach_node(PCP_CONNECTION *frontend,char *buf)
 			(errmsg("PCP: processing attach node"),
 			 errdetail("attaching Node ID %d", node_id)));
 
-	send_failback_request(node_id,true);
+	send_failback_request(node_id,true, false);
 
 	pcp_write(frontend, "c", 1);
 	wsize = htonl(sizeof(code) + sizeof(int));
@@ -934,19 +934,19 @@ process_recovery_request(PCP_CONNECTION *frontend,char *buf)
 
 	if ((!REPLICATION &&
 		 !(MASTER_SLAVE &&
-		   !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP))) ||
+		   pool_config->master_slave_sub_mode == STREAM_MODE)) ||
 		(MASTER_SLAVE &&
-		 !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+		 pool_config->master_slave_sub_mode == STREAM_MODE &&
 		 node_id == PRIMARY_NODE_ID))
 	{
-		if (MASTER_SLAVE && !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP))
+		if (MASTER_SLAVE && pool_config->master_slave_sub_mode == STREAM_MODE)
 			ereport(ERROR,
 				(errmsg("process recovery request failed"),
 					 errdetail("primary server cannot be recovered by online recovery.")));
 		else
 			ereport(ERROR,
 				(errmsg("process recovery request failed"),
-					 errdetail("recovery request is accepted only in replication mode or stereaming replication mode.")));
+					 errdetail("recovery request is only allowed in replication and streaming replication modes.")));
 	}
 	else
 	{
@@ -1051,7 +1051,7 @@ process_promote_node(PCP_CONNECTION *frontend, char *buf, char tos)
 				(errmsg("could not process recovery request"),
 				 errdetail("node id %d is not valid", node_id)));
 	/* promoting node is reserved to Streaming Replication */
-	if (!MASTER_SLAVE || (strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) != 0))
+	if (!MASTER_SLAVE || pool_config->master_slave_sub_mode != STREAM_MODE)
 	{
 		ereport(FATAL,
 			(errmsg("invalid pgpool mode for process recovery request"),
