@@ -84,11 +84,7 @@ int pool_init_cp(void)
 */
 POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, int check_socket)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	int i, freed = 0;
 	ConnectionInfo *info;
@@ -171,6 +167,7 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, in
 					info = connection_pool->info;
 					memset(connection_pool, 0, sizeof(POOL_CONNECTION_POOL));
 					connection_pool->info = info;
+					info->swallow_termination = 0;
 					memset(connection_pool->info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 					POOL_SETMASK(&oldmask);
 					return NULL;
@@ -662,7 +659,7 @@ static bool connect_with_timeout(int fd, struct addrinfo *walk, char *host, int
 					if (error != 0)
 					{
 						ereport(LOG,
-								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() detected error \"%s\"",host,port,strerror(errno))));
+								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() detected error \"%s\"",host,port,strerror(error))));
 						return false;
 					}
 				}
@@ -777,7 +774,6 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 		if (!connect_with_timeout(fd, walk, host, port, retry))
 		{
 			close(fd);
-			fd = -1;
 			continue;
 		}
 
@@ -816,8 +812,8 @@ static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int s
 }
 
 /*
- * create actual connections to backends
- * new connection resides in TopMemoryContext
+ * Create actual connections to backends.
+ * New connection resides in TopMemoryContext.
  */
 static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 {
@@ -847,19 +843,44 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 		if (create_cp(s, i) == NULL)
 		{
 			/* If fail_over_on_backend_error is true, do failover.
-			 * Otherwise, just exit this session.
+			 * Otherwise, just exit this session or skip next health node.
 			 */
 			if (pool_config->fail_over_on_backend_error)
 			{
-				notice_backend_error(i);
+				notice_backend_error(i, true);
+				ereport(FATAL,
+					(errmsg("failed to create a backend connection"),
+						 errdetail("executing failover on backend")));
 			}
 			else
 			{
-				ereport(LOG,
-					(errmsg("creating new connection to backend"),
-						 errdetail("not executing failover because fail_over_on_backend_error is off")));
+				/*
+				 * If we are in streaming replication mode and the node is a
+				 * standby node, then we skip this node to avoid fail over.
+				 */
+				if (STREAM && !IS_PRIMARY_NODE_ID(i))
+				{
+					ereport(LOG,
+							(errmsg("failed to create a backend %d connection", i),
+							 errdetail("skip this backend because because fail_over_on_backend_error is off and we are in streaming replication mode and node is standby node")));
+
+					/* set down status to local status area */
+					*(my_backend_status[i]) = CON_DOWN;
+
+					/* make sure that we need to restart the process after
+					 * finishing this session
+					 */
+					pool_get_my_process_info()->need_to_restart = 1;
+					continue;
+				}
+				else
+				{
+					ereport(FATAL,
+							(errmsg("failed to create a backend %d connection", i),
+							 errdetail("not executing failover because fail_over_on_backend_error is off")));
+				}
 			}
-			child_exit(1);
+			child_exit(POOL_EXIT_AND_RESTART);
 		}
 
 		p->info[i].create_time = time(NULL);
