@@ -5,7 +5,7 @@
 source $TESTLIBS
 TESTDIR=testdir
 PG_CTL=$PGBIN/pg_ctl
-PSQL=$PGBIN/psql
+PSQL="$PGBIN/psql -X "
 
 for mode in s r
 do
@@ -27,10 +27,11 @@ do
 	# trrigger failover
 	$PG_CTL -D data1 -m f stop
 	wait_for_pgpool_startup
-	$PSQL -c "show pool_nodes" test > result
+	$PSQL -c "show pool_nodes" test |sed -e 's/true /false/'> result
 
 	# check the output of "show pool_nodes".
-	cmp result ../expected.$mode > /dev/null 2>&1
+	LANG=C $PSQL -f ../create_expected.sql -v mode="'$mode'" -v dir="'$PGSOCKET_DIR'" test | tail -n 6 > expected
+	cmp result expected > /dev/null 2>&1
 	if [ $? != 0 ];then
 		./shutdownall
 		exit 1
