@@ -5,7 +5,7 @@
  *
  * Because of the extremely high rate at which log messages can be generated,
  * we need to be mindful of the performance cost of obtaining any information
- * that may be logged.	Also, it's important to keep in mind that this code may
+ * that may be logged.  Also, it's important to keep in mind that this code may
  * get called from within an aborted transaction, in which case operations
  * such as syscache lookups are unsafe.
  *
@@ -15,12 +15,12 @@
  * if we run out of memory, it's important to be able to report that fact.
  * There are a number of considerations that go into this.
  *
- * First, distinguish between re-entrant use and actual recursion.	It
+ * First, distinguish between re-entrant use and actual recursion.  It
  * is possible for an error or warning message to be emitted while the
  * parameters for an error message are being computed.	In this case
  * errstart has been called for the outer message, and some field values
- * may have already been saved, but we are not actually recursing.	We handle
- * this by providing a (small) stack of ErrorData records.	The inner message
+ * may have already been saved, but we are not actually recursing.  We handle
+ * this by providing a (small) stack of ErrorData records.  The inner message
  * can be computed and sent without disturbing the state of the outer message.
  * (If the inner message is actually an error, this isn't very interesting
  * because control won't come back to the outer message generator ... but
@@ -43,8 +43,8 @@
  * overflow.)
  *
  *
- * Portions Copyright (c) 2003-2014, PgPool Global Development Group
- * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2015, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2015, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -93,12 +93,15 @@ struct ONEXIT on_exit_prepare = {NULL,0L};
  * (or in the parent postmaster).
  */
 static bool atexit_callback_setup = false;
+
 /*
  * This flag is set during proc_exit() to change ereport()'s behavior,
  * so that an ereport() from an on_proc_exit routine cannot get us out
  * of the exit procedure.  We do NOT want to go back to the idle loop...
  */
+
 bool		proc_exit_inprogress = false;
+
 /* local functions */
 static void proc_exit_prepare(int code);
 
@@ -178,7 +181,7 @@ static int	frontend_error_recursion_depth = 0;	/* to detect recursion in deliver
 
 static char *expand_fmt_string(const char *fmt, ErrorData *edata);
 static const char *useful_strerror(int errnum);
-static const char *error_severity(int elevel);
+static const char *error_severity(int elevel, bool for_frontend);
 static const char *process_name(void);
 static void append_with_tabs(StringInfo buf, const char *str);
 static bool is_log_level_output(int elevel, int log_min_level);
@@ -217,12 +220,18 @@ errstart(int elevel, const char *filename, int lineno,
 	bool		output_to_server;
 	bool		output_to_client = false;
 	int			i;
-
+	int			frontend_invalid = false;
 	/*
 	 * Check some cases in which we want to promote an error into a more
 	 * severe error.  None of this logic applies for non-error messages.
 	 */
-	if (elevel >= ERROR)
+	if (elevel == FRONTEND_ERROR)
+	{
+		frontend_invalid = true;
+		elevel = ERROR;
+	}
+
+	if (elevel >= ERROR && elevel != FRONTEND_ONLY_ERROR)
 	{
 		/*
 		 * Check reasons for treating ERROR as FATAL:
@@ -264,6 +273,7 @@ errstart(int elevel, const char *filename, int lineno,
 	/* Determine whether message is enabled for server log output */
 	output_to_server = is_log_level_output(elevel, pool_config->log_min_messages);
 
+
 	/* Determine whether message is enabled for client output */
 	if (elevel != COMMERROR)
 	{
@@ -274,7 +284,7 @@ errstart(int elevel, const char *filename, int lineno,
 		 * during authentication.
 		 */
         output_to_client = (elevel >= pool_config->client_min_messages ||
-								elevel == INFO);
+								elevel == INFO || elevel == FRONTEND_ONLY_ERROR);
 	}
 
 	/* Skip processing effort if non-error message will not be output */
@@ -285,7 +295,7 @@ errstart(int elevel, const char *filename, int lineno,
 	 * Okay, crank up a stack entry to store the info in.
 	 */
 
-	if (recursion_depth++ > 0 && elevel >= ERROR)
+	if (recursion_depth++ > 0 && elevel >= ERROR && elevel != FRONTEND_ONLY_ERROR)
 	{
 		/*
 		 * Ooops, error during error processing.  Clear ErrorContext as
@@ -321,12 +331,13 @@ errstart(int elevel, const char *filename, int lineno,
 	edata = &errordata[errordata_stack_depth];
 	MemSet(edata, 0, sizeof(ErrorData));
 	edata->elevel = elevel;
+	edata->frontend_invalid = frontend_invalid;
 	edata->output_to_server = output_to_server;
 	edata->output_to_client = output_to_client;
 	if(elevel == FATAL && PG_exception_stack == NULL) /* This is startup failure. Take down main process with it */
 		edata->retcode = POOL_EXIT_FATAL;
 	else
-		edata->retcode = POOL_EXIT_NOFATAL;
+		edata->retcode = POOL_EXIT_AND_RESTART;
 	if (filename)
 	{
 		const char *slash;
@@ -340,7 +351,7 @@ errstart(int elevel, const char *filename, int lineno,
 	edata->lineno = lineno;
 	edata->funcname = funcname;
 	/* the default text domain is the backend's */
-	edata->domain = domain ? domain : PG_TEXTDOMAIN("postgres");
+	edata->domain = domain ? domain : PG_TEXTDOMAIN("pgpool");
 
     edata->saved_errno = errno;
 
@@ -481,7 +492,7 @@ errfinish(int dummy,...)
 		proc_exit(retcode);
 	}
 
-	if (elevel >= PANIC)
+	if (elevel >= PANIC && elevel != FRONTEND_ONLY_ERROR)
 	{
 		/*
 		 * Serious crash time. Postmaster will observe SIGABRT process exit
@@ -533,15 +544,6 @@ int pool_error_code(const char *errcode)
 	return 0;					/* return value does not matter */
 }
 
-/*
- * errcode_for_socket_access --- add SQLSTATE error code to the current error
- *
- * The SQLSTATE code is chosen based on the saved errno value.	We assume
- * that the failing operation was some type of socket access.
- *
- * NOTE: the primary error message string should generally include %m
- * when this is used.
- */
 
 /*
  * This macro handles expansion of a format string and associated parameters;
@@ -573,13 +575,13 @@ int pool_error_code(const char *errcode)
 		for (;;) \
 		{ \
 			va_list		args; \
-			bool		success; \
+			int			needed; \
 			va_start(args, fmt); \
-			success = appendStringInfoVA(&buf, fmtbuf, args); \
+			needed = appendStringInfoVA(&buf, fmtbuf, args); \
 			va_end(args); \
-			if (success) \
+			if (needed == 0) \
 				break; \
-			enlargeStringInfo(&buf, buf.maxlen); \
+			enlargeStringInfo(&buf, needed); \
 		} \
 		/* Done with expanded fmt */ \
 		pfree(fmtbuf); \
@@ -616,13 +618,13 @@ int pool_error_code(const char *errcode)
 		for (;;) \
 		{ \
 			va_list		args; \
-			bool		success; \
+			int			needed; \
 			va_start(args, n); \
-			success = appendStringInfoVA(&buf, fmtbuf, args); \
+			needed = appendStringInfoVA(&buf, fmtbuf, args); \
 			va_end(args); \
-			if (success) \
+			if (needed == 0) \
 				break; \
-			enlargeStringInfo(&buf, buf.maxlen); \
+			enlargeStringInfo(&buf, needed); \
 		} \
 		/* Done with expanded fmt */ \
 		pfree(fmtbuf); \
@@ -968,6 +970,21 @@ geterrcode(void)
 	return edata->sqlerrcode;
 }
 
+/*
+ * getfrontendinvalid --- return the currently frontend_invalid value
+ *
+ * This is only intended for use in error callback subroutines, since there
+ * is no other place outside elog.c where the concept is meaningful.
+ */
+bool getfrontendinvalid(void)
+{
+	ErrorData  *edata = &errordata[errordata_stack_depth];
+
+	/* we don't bother incrementing recursion_depth */
+	CHECK_STACK_DEPTH();
+
+	return edata->frontend_invalid;
+}
 /*
  * geterrposition --- return the currently set error position (0 if none)
  *
@@ -1463,7 +1480,7 @@ write_syslog(int level, const char *line)
 	/* Open syslog connection if not done yet */
 	if (!openlog_done)
 	{
-		openlog(pool_config->syslog_ident ? pool_config->syslog_ident : "pgpool-II",
+		openlog(pool_config->syslog_ident ? pool_config->syslog_ident : "pgpool",
 				LOG_PID | LOG_NDELAY | LOG_NOWAIT,
 				pool_config->syslog_facility);
 		openlog_done = true;
@@ -1580,7 +1597,7 @@ write_eventlog(int level, const char *line, int len)
 
 	if (evtHandle == INVALID_HANDLE_VALUE)
 	{
-		evtHandle = RegisterEventSource(NULL, event_source ? event_source : "PostgreSQL");
+		evtHandle = RegisterEventSource(NULL, event_source ? event_source : "pgpool");
 		if (evtHandle == NULL)
 		{
 			evtHandle = INVALID_HANDLE_VALUE;
@@ -1729,15 +1746,9 @@ write_console(const char *line, int len)
 static void
 send_message_to_frontend(ErrorData *edata)
 {
-	int protoVersion;
+	int protoVersion = PROTO_MAJOR_V3; /* default protocol version is V3 also used by pcp lib */
 
-	if (pg_frontend_exists() < 0 )
-		return;
-
-	/*
-	 * Do not forward the debug messages to client before session is initialized
-	 */
-	if (edata->elevel < ERROR && pool_get_session_context(true) == NULL)
+	if (pool_frontend_exists() < 0 )
 		return;
 	/*
 	 * Leave if we are failing on sending the message to frontend.
@@ -1745,16 +1756,28 @@ send_message_to_frontend(ErrorData *edata)
 	if (++frontend_error_recursion_depth > 2)
 		return;
 
-	protoVersion = get_frontend_protocol_version();
-	set_pg_frontend_blocking(false);
+	if(processType == PT_CHILD)
+	{
+		/*
+		 * Do not forward the debug messages to client before session is initialized
+		 */
+		if (edata->elevel < ERROR && pool_get_session_context(true) == NULL)
+		{
+			frontend_error_recursion_depth--;
+			return;
+		}
+		protoVersion = get_frontend_protocol_version();
+		set_pg_frontend_blocking(false);
+	}
 
 	if (protoVersion == PROTO_MAJOR_V2)
 	{
 		char* message = edata->message?edata->message:"missing error text";
-		send_to_pg_frontend((edata->elevel < ERROR) ? "N" : "E", 1,false);
-		send_to_pg_frontend(message, strlen(message),false);
-		send_to_pg_frontend("\n", 1,true);
+		pool_send_to_frontend((edata->elevel < ERROR) ? "N" : "E", 1,false);
+		pool_send_to_frontend(message, strlen(message),false);
+		pool_send_to_frontend("\n", 1,true);
 	}
+
 	else if (protoVersion == PROTO_MAJOR_V3)
 	{
         /*
@@ -1773,13 +1796,13 @@ send_message_to_frontend(ErrorData *edata)
 		int len;
 		int thislen;
 		int sendlen;
-
+        
 		len = 0;
 		memset(data, 0, MAXDATA);
-		send_to_pg_frontend((edata->elevel < ERROR) ? "N" : "E", 1, false);
-        
+		pool_send_to_frontend((edata->elevel < ERROR) ? "N" : "E", 1, false);
+
 		/* error level */
-		thislen = snprintf(msgbuf, MAXMSGBUF, "S%s", error_severity(edata->elevel));
+		thislen = snprintf(msgbuf, MAXMSGBUF, "S%s", error_severity(edata->elevel,true));
 		thislen = Min(thislen, MAXMSGBUF);
 		memcpy(data +len, msgbuf, thislen+1);
 		len += thislen + 1;
@@ -1833,11 +1856,29 @@ send_message_to_frontend(ErrorData *edata)
 		sendlen = len;
 		len = htonl(len + 4);
 
-		send_to_pg_frontend((char*)&len, sizeof(len), false);
-		send_to_pg_frontend(data, sendlen, true);
+		pool_send_to_frontend((char*)&len, sizeof(len), false);
+		pool_send_to_frontend(data, sendlen, true);
+
+	}
+	if (edata->elevel == FRONTEND_ONLY_ERROR)
+	{
+		/* send the ready for query to complete
+		 * the frontend error cycle
+		 */
+		/* ready for query */
+		pool_send_to_frontend((char*)"Z", 1, true);
+
+		if (protoVersion == PROTO_MAJOR_V3)
+		{
+			int len = htonl(5);
+			pool_send_to_frontend((char*)&len, sizeof(len), false);
+			pool_send_to_frontend((char*)"I", 1, true);
+		}
 	}
 
-	set_pg_frontend_blocking(true);
+	if (processType == PT_CHILD)
+		set_pg_frontend_blocking(true);
+
 	frontend_error_recursion_depth--;
 }
 
@@ -2041,7 +2082,7 @@ send_message_to_server_log(ErrorData *edata)
 	initStringInfo(&buf);
 
 	log_line_prefix(&buf, pool_config->log_line_prefix, edata);
-	appendStringInfo(&buf, "%s:  ", error_severity(edata->elevel));
+	appendStringInfo(&buf, "%s:  ", error_severity(edata->elevel,false));
 
 
 	if (edata->message)
@@ -2116,7 +2157,7 @@ send_message_to_server_log(ErrorData *edata)
 
 #ifdef HAVE_VSYSLOG
 	/* Write to syslog, if enabled */
-    if (pool_config->logsyslog == 1)
+	if (pool_config->log_destination & LOG_DESTINATION_SYSLOG)
     {
 		int			syslog_level;
 
@@ -2139,6 +2180,7 @@ send_message_to_server_log(ErrorData *edata)
 				syslog_level = LOG_NOTICE;
 				break;
 			case ERROR:
+			case FRONTEND_ONLY_ERROR:
 				syslog_level = LOG_WARNING;
 				break;
 			case FATAL:
@@ -2149,12 +2191,14 @@ send_message_to_server_log(ErrorData *edata)
 				syslog_level = LOG_CRIT;
 				break;
 		}
-		if (pool_config->logsyslog == 1)
-			write_syslog(syslog_level, buf.data);
+		write_syslog(syslog_level, buf.data);
 	}
 #endif   /* HAVE_VSYSLOG */
 
-    write_console(buf.data, buf.len);
+	if (pool_config->log_destination & LOG_DESTINATION_STDERR)
+	{
+		write_console(buf.data, buf.len);
+	}
 	pfree(buf.data);
 }
 
@@ -2251,7 +2295,7 @@ useful_strerror(int errnum)
  * error_severity --- get localized string representing elevel
  */
 static const char *
-error_severity(int elevel)
+error_severity(int elevel, bool for_frontend)
 {
 	const char *prefix;
 
@@ -2277,6 +2321,12 @@ error_severity(int elevel)
 		case WARNING:
 			prefix = _("WARNING");
 			break;
+		case FRONTEND_ONLY_ERROR:
+			if (for_frontend == false)
+				prefix = _("FRONTEND_ERROR");
+			else
+				prefix = _("ERROR");
+			break;
 		case ERROR:
 			prefix = _("ERROR");
 			break;
@@ -2335,6 +2385,9 @@ process_name(void)
 		case PT_PCP:
 			prefix = _("PCP CHILD");
 			break;
+		case PT_PCP_WORKER:
+			prefix = _("PCP WORKER");
+			break;
 		case PT_HB_SENDER:
 			prefix = _("WD HB SENDER");
 			break;
@@ -2410,13 +2463,13 @@ write_stderr(const char *fmt,...)
  *
  * We use this for tests that should consider LOG to sort out-of-order,
  * between ERROR and FATAL.  Generally this is the right thing for testing
- * whether a message should go to the postmaster log, whereas a simple >=
+ * whether a message should go to the pgpool log, whereas a simple >=
  * test is correct for testing whether the message should go to the client.
  */
 static bool
 is_log_level_output(int elevel, int log_min_level)
 {
-	if (elevel == LOG || elevel == COMMERROR)
+	if (elevel == LOG || elevel == COMMERROR || elevel == FRONTEND_ONLY_ERROR)
 	{
 		if (log_min_level == LOG || log_min_level <= ERROR)
 			return true;
