@@ -44,7 +44,6 @@ static bool is_immutable_function(char *fname);
 static bool select_table_walker(Node *node, void *context);
 static bool non_immutable_function_call_walker(Node *node, void *context);
 static char *strip_quote(char *str);
-static char *make_table_name_from_rangevar(RangeVar *rangevar);
 
 /*
  * Return true if this SELECT has function calls *and* supposed to
@@ -59,12 +58,32 @@ bool pool_has_function_call(Node *node)
 		return false;
 
 	ctx.has_function_call = false;
+	ctx.pg_terminate_backend_pid = -1;
 
 	raw_expression_tree_walker(node, function_call_walker, &ctx);
 
 	return ctx.has_function_call;
 }
 
+/*
+ * Search the pg_terminate_backend() call in the query
+ */
+int pool_get_terminate_backend_pid(Node *node)
+{
+	SelectContext	ctx;
+
+	if (!IsA(node, SelectStmt))
+		return false;
+
+	ctx.has_function_call = false;
+	ctx.pg_terminate_backend_pid = 0;
+
+	raw_expression_tree_walker(node, function_call_walker, &ctx);
+
+	return ctx.pg_terminate_backend_pid;
+
+}
+
 /*
  * Return true if this SELECT has system catalog table.
  */
@@ -168,6 +187,8 @@ bool pool_has_insertinto_or_locking_clause(Node *node)
 int pattern_compare(char *str, const int type, const char *param_name)
 {
 	int i = 0;
+	char *s;
+	int result = 0;
 
 	RegPattern *lists_patterns;
 	int *pattc;
@@ -190,11 +211,12 @@ int pattern_compare(char *str, const int type, const char *param_name)
 		return -1;
 	}
 
+	s = strip_quote(str);
 	for (i = 0; i < *pattc; i++) {
 		if (lists_patterns[i].type != type)
 			continue;
 
-		if (regexec(&lists_patterns[i].regexv, str, 0, 0, 0) == 0)
+		if (regexec(&lists_patterns[i].regexv, s, 0, 0, 0) == 0)
 		{
 			switch(type) {
 			/* return 1 if string matches whitelist pattern */
@@ -202,29 +224,55 @@ int pattern_compare(char *str, const int type, const char *param_name)
 				ereport(DEBUG2,
 					(errmsg("comparing function name in whitelist regex array"),
 						errdetail("pattern_compare: %s (%s) matched: %s",
-							   param_name, lists_patterns[i].pattern, str)));
-				return 1;
+							   param_name, lists_patterns[i].pattern, s)));
+				result = 1;
+				break;
 			/* return 1 if string matches blacklist pattern */
 			case BLACKLIST:
 				ereport(DEBUG2,
 					(errmsg("comparing function name in blacklist regex array"),
 						 errdetail("pattern_compare: %s (%s) matched: %s",
-								   param_name, lists_patterns[i].pattern, str)));
-				return 1;
+								   param_name, lists_patterns[i].pattern, s)));
+				result = 1;
+				break;
 			default:
 				ereport(WARNING,
-						(errmsg("pattern_compare: \"%s\" unknown pattern match type: \"%s\"", param_name, str)));
-				return -1;
+						(errmsg("pattern_compare: \"%s\" unknown pattern match type: \"%s\"", param_name, s)));
+				result = -1;
+				break;
 			}
+			/* return the result */
+			break;
 		}
 		ereport(DEBUG2,
 			(errmsg("comparing function name in blacklist/whitelist regex array"),
 				 errdetail("pattern_compare: %s (%s) not matched: %s",
-						   param_name, lists_patterns[i].pattern, str)));
+						   param_name, lists_patterns[i].pattern, s)));
 	}
 
-	/* return 0 otherwise */
-	return 0;
+	free(s);
+	return result;
+}
+
+static char *strip_quote(char *str)
+{
+	char *after;
+	int i = 0;
+
+	after = malloc(sizeof(char) * strlen(str) + 1);
+
+	do {
+		if (*str != '"')
+		{
+			after[i] = *str;
+			i++;
+		}
+		str++;
+	} while (*str != '\0');
+
+	after[i] = '\0';
+
+	return after;
 }
 
 /*
@@ -258,6 +306,20 @@ static bool function_call_walker(Node *node, void *context)
 			ereport(DEBUG1,
 				(errmsg("function call walker, function name: \"%s\"", fname)));
 
+			if (ctx->pg_terminate_backend_pid == 0 && strcmp("pg_terminate_backend", fname) == 0)
+			{
+				if (list_length(fcall->args) == 1)
+				{
+					Node *arg = linitial(fcall->args);
+					if (IsA(arg, A_Const) &&
+					   ((A_Const *)arg)->val.type == T_Integer)
+					{
+						ctx->pg_terminate_backend_pid = ((A_Const *)arg)->val.val.ival;
+						ereport(DEBUG1,
+								(errmsg("pg_terminate_backend pid = %d",ctx->pg_terminate_backend_pid)));
+					}
+				}
+			}
 			/*
 			 * Check white list if any.
 			 */
@@ -433,9 +495,12 @@ static bool is_system_catalog(char *table_name)
 
 	if (table_name == NULL)
 	{
-			return false;
+		return false;
 	}
 
+	if (!pool_has_to_regclass() && !pool_has_pgpool_regclass())
+		table_name = remove_quotes_and_schema_from_relname(table_name);
+
 	backend = pool_get_session_context(false)->backend;
 
 	/*
@@ -623,11 +688,11 @@ bool is_unlogged_table(char *table_name)
  * Query to know if the target table is a unlogged one.  This query
  * is valid in PostgreSQL 9.1 or later.
  */
-#define ISUNLOGGEDQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.relname = '\"%s\"' AND c.relpersistence = 'u'"
+#define ISUNLOGGEDQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.relname = '%s' AND c.relpersistence = 'u'"
 
-#define ISUNLOGGEDQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('\"%s\"') AND c.relpersistence = 'u'"
+#define ISUNLOGGEDQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('%s') AND c.relpersistence = 'u'"
 
-#define ISUNLOGGEDQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = to_regclass('\"%s\"') AND c.relpersistence = 'u'"
+#define ISUNLOGGEDQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = to_regclass('%s') AND c.relpersistence = 'u'"
 
 	int hasrelpersistence;
 	static POOL_RELCACHE *hasrelpersistence_cache;
@@ -636,9 +701,12 @@ bool is_unlogged_table(char *table_name)
 
 	if (table_name == NULL)
 	{
-			return false;
+		return false;
 	}
 
+	if (!pool_has_to_regclass() && !pool_has_pgpool_regclass())
+		table_name = remove_quotes_and_schema_from_relname(table_name);
+
 	backend = pool_get_session_context(false)->backend;
 
 	/*
@@ -716,9 +784,9 @@ bool is_view(char *table_name)
  */
 #define ISVIEWQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.relname = '%s' AND (c.relkind = 'v' OR c.relkind = 'm')"
 
-#define ISVIEWQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('\"%s\"') AND (c.relkind = 'v' OR c.relkind = 'm')"
+#define ISVIEWQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = pgpool_regclass('%s') AND (c.relkind = 'v' OR c.relkind = 'm')"
 
-#define ISVIEWQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = to_regclass('\"%s\"') AND (c.relkind = 'v' OR c.relkind = 'm')"
+#define ISVIEWQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c WHERE c.oid = to_regclass('%s') AND (c.relkind = 'v' OR c.relkind = 'm')"
 
 	static POOL_RELCACHE *relcache;
 	POOL_CONNECTION_POOL *backend;
@@ -727,9 +795,12 @@ bool is_view(char *table_name)
 
 	if (table_name == NULL)
 	{
-			return false;
+		return false;
 	}
 
+	if (!pool_has_to_regclass() && !pool_has_pgpool_regclass())
+		table_name = remove_quotes_and_schema_from_relname(table_name);
+
 	backend = pool_get_session_context(false)->backend;
 
 	/* PostgreSQL 9.4 or later has to_regclass() */
@@ -982,9 +1053,9 @@ int pool_table_name_to_oid(char *table_name)
 /*
  * Query to convert table name to oid
  */
-#define TABLE_TO_OID_QUERY "SELECT pgpool_regclass('\"%s\"')"
-#define TABLE_TO_OID_QUERY2 "SELECT oid FROM pg_class WHERE relname = '\"%s\"'"
-#define TABLE_TO_OID_QUERY3 "SELECT COALESCE(to_regclass('\"%s\"')::oid, 0)"
+#define TABLE_TO_OID_QUERY "SELECT pgpool_regclass('%s')"
+#define TABLE_TO_OID_QUERY2 "SELECT oid FROM pg_class WHERE relname = '%s'"
+#define TABLE_TO_OID_QUERY3 "SELECT COALESCE(to_regclass('%s')::oid, 0)"
 
 	int oid = 0;
 	static POOL_RELCACHE *relcache;
@@ -996,6 +1067,9 @@ int pool_table_name_to_oid(char *table_name)
 		return oid;
 	}
 
+	if (!pool_has_to_regclass() && !pool_has_pgpool_regclass())
+		table_name = remove_quotes_and_schema_from_relname(table_name);
+
 	backend = pool_get_session_context(false)->backend;
 
 	if (pool_has_to_regclass())
@@ -1075,7 +1149,6 @@ select_table_walker(Node *node, void *context)
 		RangeVar *rgv = (RangeVar *)node;
 		char *table;
 		int oid;
-		char *s;
 
 		table = make_table_name_from_rangevar(rgv);
 		oid = pool_table_name_to_oid(table);
@@ -1094,9 +1167,7 @@ select_table_walker(Node *node, void *context)
 			num_oids = ctx->num_oids++;
 
 			ctx->table_oids[num_oids] = oid;
-			s = strip_quote(table);
-			strlcpy(ctx->table_names[num_oids], s, POOL_NAMEDATALEN);
-			free(s);
+			strlcpy(ctx->table_names[num_oids], table, POOL_NAMEDATALEN);
 
 			ereport(DEBUG1,
 				(errmsg("extracting table oids from SELECT statement"),
@@ -1108,26 +1179,6 @@ select_table_walker(Node *node, void *context)
 	return raw_expression_tree_walker(node, select_table_walker, context);
 }
 
-static char *strip_quote(char *str)
-{
-	char *after;
-	int i = 0;
-
-	after = malloc(sizeof(char) * strlen(str) + 1);
-
-	do {
-		if (*str != '"')
-		{
-			after[i] = *str;
-			i++;
-		}
-		str++;
-	} while (*str != '\0');
-
-	after[i] = '\0';
-
-	return after;
-}
 
 /*
  * makeRangeVarFromNameList
@@ -1168,16 +1219,18 @@ makeRangeVarFromNameList(List *names)
  * necessary.  The returned table name is in static area. So next
  * call to this function will break previous result.
  */
-static char *make_table_name_from_rangevar(RangeVar *rangevar)
+char *make_table_name_from_rangevar(RangeVar *rangevar)
 {
 	/*
 	 * Table name. Max size is calculated as follows:
 	 * schema name(POOL_NAMEDATALEN byte)
-	 * + single quote(1 byte)
+	 * + quotation marks for schmea name(2 byte)
+	 * + period(1 byte)
 	 * + table name (POOL_NAMEDATALEN byte)
+	 * + quotation marks for table name(2 byte)
 	 * + NULL(1 byte)
 	 */
-	static char tablename[POOL_NAMEDATALEN*2+1+1];
+	static char tablename[POOL_NAMEDATALEN*2+1+2*2+1];
 
 	if (rangevar == NULL)
 	{
@@ -1198,7 +1251,9 @@ static char *make_table_name_from_rangevar(RangeVar *rangevar)
 
 	if (rangevar->schemaname)
 	{
-		strncpy(tablename, rangevar->schemaname, POOL_NAMEDATALEN);
+		strcat(tablename, "\"");
+		strncat(tablename, rangevar->schemaname, POOL_NAMEDATALEN);
+		strcat(tablename, "\"");
 		strcat(tablename, ".");
 	}
 
@@ -1210,8 +1265,12 @@ static char *make_table_name_from_rangevar(RangeVar *rangevar)
 		return "";
 	}
 
+	strcat(tablename, "\"");
 	strncat(tablename, rangevar->relname, POOL_NAMEDATALEN);
+	strcat(tablename, "\"");
+
 	ereport(DEBUG1,
 			(errmsg("make table name from rangevar: tablename:\"%s\"", tablename)));
+
 	return tablename;
 }
